import time
import requests
from typing import Union, Literal

from . import defs

def get_ngs_url(lat: float, lon: float, ngs_model: int):
    """
    :param float lat: Decimal latitude
    :param float lon: Decimal longitude
    :param str ngs_model: The NGS geoid model to use for lookup `(see list) <https://www.ngs.noaa.gov/web_services/geoid.shtml>`_
    :return: The query url to use in lookup
    :rtype: str
    """
    return defs.NGS_URL % (lat, lon, ngs_model)

def get_ngs_json(ngs_url: str):
    """
    :param str ngs_url: Decimal latitude
    :return: The returned json (if applicable)
    :rtype: json
    """
    i = 0
    while True:
        print('Querying %s' % (ngs_url))
        response = requests.get(ngs_url)
        json_data = response.json() if response and response.status_code == 200 else None
        if json_data and 'geoidHeight' in json_data:
            return json_data
        if json_data and (not 'geoidHeight' in json_data):
            print('Json data request returned in error: %s' % (json_data))
            exit(1)
        if i < 3:
            i += 1
            time.sleep(1)
        else:
            print('Could not complete request for NGS API in %s tries.' % (i  ))
            exit(1)

def get_vdatum_url(lat: float, lon: float, vdatum_model: str, region: str):
    """
    .. |vdatum| raw:: html

        <a href="https://vdatum.noaa.gov/docs/services.html#step160" target="_blank">see list</a>

    
    :param float lat: Decimal latitude
    :param float lon: Decimal longitude
    :param str vdatum_model: The VDatum geoid, tidal, or potential model to use for lookup (|vdatum|)
    :param str region: The region to search
    """
    wgs = 'WGS84_G1674'
    return defs.VDATUM_URL % (
            lon, # s_x
            lat, # s_y
            wgs, # s_h_frame
            vdatum_model, # s_v_frame
            vdatum_model, # s_v_geoid
            wgs, # t_h_frame
            wgs, # t_v_frame
            vdatum_model, # t_v_geoid
            region # region
            )

def get_vdatum_json(vdatum_url, region):
    """
    
    :param float lat: Decimal latitude
    :param float lon: Decimal longitude
    :param str vdatum_model: The VDatum geoid, tidal, or potential model to use for lookup (|vdatum|)
    :param str region: The region to search
    """
    while True:
        print('Querying %s' % (vdatum_url))
        response = requests.get(vdatum_url)
        json_data = response.json() if response and response.status_code == 200 else None
        if json_data and ('t_z' in json_data):
            return json_data
        if json_data and ('errorCode' in json_data):
            if 'Selected Region is Invalid!' in json_data['message']:
                raise AttributeError('Region "%s" is not valid!' % (region))
            else:
                raise AttributeError('VDatum API error %s: %s' % (json_data['errorCode'], json_data['message']))

"""def adjustment(user_vrs: Union[str, Literal[None]]=None,
               las_vrs: Union[str, Literal[None]]=None, # overrides user_vrs.
               # consequently implies we trust file headers;
               # this is done to support projects with multiple CRS
               # and to enforce correct CRS info in database
               lat:float=0.0, lon: float=0.0,
               region=defs.REGIONS[0]):
    ### docstring start
    Get the geoid, tidal, or geopotential model height above the ellipsoid
    in order to convert model-referenced heights to ellipsoid height (i.e.
    compatible with Cesium) and vice-versa.

    The following figure demonstrates the difference between geoid, ellipsoid,
    and topographic ground surface:

    .. figure:: https://user-images.githubusercontent.com/18689918/239385604-5b5dd0df-e2fb-4ea9-90e7-575287a069e6.png
        :align: center

        Diagram showing conceptual model of ellipsoid height ``h``, geoid
        height ``H``, and height of geoid relative to ellipsoid ``N``
        along with topographic surface (grey).

    Ellipsoidal height (``h``) is generally used in global projections such as
    Cesium due to its small digital footprint and ease of calculation relative
    to systems based on gravity or geopotential height. However, the earth and
    tides are influenced by local differences in gravitational pull and other
    factors. Therefore some engineering projects and local reference systems
    use height referenced to a geoid or tidal model (``H``) which provides a much
    easier framework to understand height relative to, for example, modeled
    mean sea level or sea level potential. Converting from ``H`` to ``h``
    requires knowing the height difference between the geoid and the ellipsoid
    (``N``).
    Conversion is then a simple addition of these values (``H + N = h``).

    This function will use either the
    `NGS <https://www.ngs.noaa.gov/web_services/geoid.shtml>`_ or
    `VDatum https://vdatum.noaa.gov/docs/services.html`_ API to look up
    geoid height relative to the ellipsoid (`N`).

    .. note::

        ``las_vrs`` is set by file headers and overrides ``user_vrs``.
        This implicitly means we trust file headers over user input.
        We do this to support projects with multiple VRS (i.e. `user_vrs`
        values are used solely to fill in gaps where headers do not explicitly
        specify a vertical reference system). It is also meant to enforce the
        accuracy of VRS information in file headers.

        If a project should need to set or change VRS information prior to
        uploading to the database, they are encouraged to do so through the
        use of third-party software such as
        `LASTools <https://github.com/LAStools/LAStools>`_.

    :param user_vrs: The user-specified geoid model to convert from if none is found in the file header
    :type user_vrs: str or None
    :param las_vrs: WKT-derived model from file header to convert from (overrides ``user_vrs``)
    :type las_vrs: str or None
    :param float lat: Decimal latitude
    :param float lon: Decimal longitude
    :param str region: The VDatum region `(see list in docs) <https://vdatum.noaa.gov/docs/services.html#step140>`_

    :return: 
    :rtype: int or pyproj.crs.CRS
    ### docstring end

    # the 9 possible scenarios and their outcomes
    # 1. matched las_vrs / matched user_vrs -> las_vrs
    # 2. matched las_vrs / unmatched user_vrs -> las_vrs
    # 3. matched las_vrs / empty user_vrs -> las_vrs
    # 4. empty las_vrs / empty user_vrs -> 0
    # 5. empty las_vrs / matched user_vrs -> user_vrs
    # 6. empty las_vrs / unmatched user_vrs -> exit(1)
    # 7. unmatched las_vrs / empty user_vrs -> exit(1)
    # 8. unmatched las_vrs / matched user_vrs -> exit(1) # maybe in the future we have a geoid_override setting where this will execute
    # 9. unmatched las_vrs / unmatched user_vrs -> exit(1)

    vrs = None
    if las_vrs:
        # override user value with detected VRS
        vrs = model_search(las_vrs)
        if user_vrs and vrs:
            # scenarios 1 and 2
            print('User value of "%s" will be overridden by detected VRS "%s"' % (user_vrs, vrs))
        if user_vrs and (not vrs):
            # scenarios 8 and 9
            print('No vertical reference system matching "%s" found' % (user_vrs))
            exit(1)
        if not user_vrs:
            # scenario 3
            pass
        if (not user_vrs) and (not vrs):
            # scenario 7
            print('No vertical reference system matching "%s" found' % (las_vrs))
    else:
        if not user_vrs:
            # scenario 4
            return 0
        else:
            vrs = model_search(user_vrs)
            if vrs:
                # scenario 5
                print('VRS found: %s (user-specified)' % (vrs))
            else:
                # scenario 6
                print('Could not find VRS matching value "%s"' % (user_vrs))
                exit(1)

    if vrs in defs.NGS_MODELS:
        # format url for NGS API, then interpret json response
        ngs_model = defs.NGS_MODELS[vrs]
        ngs_json = get_ngs_json(lat, lon, ngs_model)
        return float(ngs_json['geoidHeight'])
    if vrs in defs.VDATUM_MODELS:
        # format url for VDatum API, then interpret json response
        vdatum_json = get_vdatum_json(lat, lon, vrs, region)
        return float(vdatum_json['t_z'])
"""



from pathlib import Path
from typing import Union, Literal
from pyproj import CRS
from math import isclose

from . import defs
from . import utils

class HeightModel:
    """
    Look up the geoid, tidal, or geopotential model height above the ellipsoid
    in order to convert model-referenced heights to ellipsoid height (i.e.
    compatible with Cesium) and vice-versa.

    The following figure demonstrates the difference between geoid, ellipsoid,
    and topographic ground surface:

    .. figure:: https://user-images.githubusercontent.com/18689918/239385604-5b5dd0df-e2fb-4ea9-90e7-575287a069e6.png
        :align: center

        Diagram showing conceptual model of ellipsoid height ``h``, geoid
        height ``H``, and height of geoid relative to ellipsoid ``N``
        along with topographic surface (grey).

    Ellipsoidal height (``h``) is generally used in global projections such as
    Cesium due to its small digital footprint and ease of calculation relative
    to systems based on gravity or geopotential height. However, gravity and
    tides are influenced by local differences in Earth's density and other
    factors. Therefore some projects prefer reference systems that use height
    referenced to a geoid or tidal model (``H``) which provides a much easier
    framework to understand height relative to, for example, modeled mean sea
    level or sea level potential. Converting from ``H`` to ``h`` requires
    knowing the height difference between the geoid and the ellipsoid (``N``).
    Conversion is then a simple addition of these values (``H + N = h``).

    The lookup performed by this :class:`pyegt.height.HeightModel` class will
    use either the `NGS <https://www.ngs.noaa.gov/web_services/geoid.shtml>`_
    or `VDatum <https://vdatum.noaa.gov/docs/services.html>`_ API to look up
    model height relative to the ellipsoid (``N``).


    :param float lat: Decimal latitude
    :param float lon: Decimal longitude
    :param region: Region (for list see :py:func:`pyegt.height.HeightModel.from_model`)
    :type region: str or None
    :param from_model: Model (for list see :py:func:`pyegt.height.HeightModel.available_regions`)
    :type from_model: str or None
    :param from_vrs: The vertical reference system to convert from (``CRS.is_vertical`` must be true for the main CRS or a sub-CRS)
    :type from_vrs: pyproj.CRS or None
    :param from_wkt: Use well-known text to get a VRS to convert from
    :type from_wkt: str or None
    :param from_epsg: Use an EPSG code to get a VRS to convert from
    :type from_epsg: str or None
    """
    def __init__(self,
                 lat: float,
                 lon: float,
                 region: Union[str, Literal[None]]=None,
                 from_model: Union[str, Literal[None]]=None,
                 from_vrs: Union[CRS, Literal[None]]=None,
                 from_wkt: Union[str, Literal[None]]=None,
                 from_epsg: Union[str, int, Literal[None]]=None,
                 ) -> None:
        self.lat = lat
        self.lon = lon
        self.vrs = from_vrs
        self.model = from_model
        self.region = region
        self.wkt = from_wkt
        self.epsg = from_epsg
        self.url = None
        self.height = None
        if from_model and (not from_wkt and not from_epsg and not from_vrs):
            self.from_model()
        elif from_vrs and (not from_wkt and not from_epsg and not from_model):
            self.from_vrs()
        elif from_wkt and (not from_vrs and not from_epsg and not from_model):
            self.from_wkt()
        elif from_epsg and (not from_vrs and not from_wkt and not from_model):
            self.from_epsg()
        elif (not from_vrs and not from_epsg and not from_wkt and not from_model):
            # starting out with a blank slate
            self.vrs = None
        else:
            raise AttributeError('Only one of the parameters [from_model, from_vrs, from_wkt, from_epsg] can be set at once to avoid collisions.')
    
    def from_model(self, model: Union[str, Literal[None]]=None):
        """
        Compute the height above the ellipsoid for the given model.
        Will set ``self.model`` if the ``model`` variable is supplied.
        Finally, it will call ``self.get_height()``.

        Valid model values are::

            # using NGS API
            ['GEOID99', 'G99SSS', 'GEOID03', 'USGG2003', 'GEOID06', 'USGG2009',
            'GEOID09', 'XUSHG', 'USGG2012', 'GEOID12A', 'GEOID12B', 'GEOID18',]

            # using VDatum API
            ['NAVD88', 'NGVD29', 'ASVD02', 'W0_USGG2012', 'GUVD04', 'NMVD03',
            'PRVD02', 'VIVD09', 'CRD', 'EGM2008', 'EGM1996', 'EGM1984',
            'XGEOID16B', 'XGEOID17B', 'XGEOID18B', 'XGEOID19B', 'XGEOID20B',
            'IGLD85', 'LWD_IGLD85', 'OHWM_IGLD85', 'CRD', 'LMSL', 'MLLW',
            'MLW', 'MTL', 'DTL', 'MHW', 'MHHW', 'LWD', 'NAD27', 'NAD83_1986',
            'NAD83_2011', 'NAD83_NSRS2007', 'NAD83_MARP00', 'NAD83_PACP00',
            'WGS84_G1674', 'ITRF2014', 'IGS14', 'ITRF2008', 'IGS08',
            'ITRF2005', 'IGS2005', 'WGS84_G1150', 'ITRF2000', 'IGS00', 'IGb00',
            'ITRF96', 'WGS84_G873', 'ITRF94', 'ITRF93', 'ITRF92', 'SIOMIT92',
            'WGS84_G730', 'ITRF91', 'ITRF90', 'ITRF89', 'ITRF88',
            'WGS84_TRANSIT', 'WGS84_G1762', 'WGS84_G2139']

        Documentation can be found here for the
        `NGS <https://www.ngs.noaa.gov/web_services/geoid.shtml>`_ and
        `VDatum <https://vdatum.noaa.gov/docs/services.html>`_ APIs.
        """
        if (not model) and (not self.model):
            raise AttributeError('Cannot calculate model height, as no model has been set.')
        if model:
            self.model = model
        self.model = self.model.upper()
        return self.get_height()
        
    def from_vrs(self, vrs: Union[CRS, Literal[None]]):
        """

        """
        if (not vrs) and (not self.vrs):
            raise AttributeError('Cannot calculate VRS height, as no VRS has been set.')
        if vrs:
            self.vrs = vrs
        if self.vrs.is_compound:
            for rs in self.vrs.sub_crs_list:
                if rs.is_vertical:
                    self.vrs = rs
        elif self.vrs.is_vertical:
            self.vrs = vrs
        return self.from_model(self.vrs.name)

    def from_wkt(self, wkt: Union[str, Literal[None]]):
        """
        """
        rs = CRS.from_wkt(wkt)
        return self.from_vrs(rs)

    def from_epsg(self, epsg: Union[str, int, Literal[None]]):
        """
        """
        rs = CRS.from_epsg(epsg)
        return self.from_vrs(rs)

    def verify_model(self) -> bool:
        """
        Search for a model in the list of known models.
        
        :param str vrs: The search term, potentially a phrase containing the model name; ex: ``EGM2008 height`` will return ``EGM2008``
        :return: Returns the correctly formatted model name to use in the API query
        :rtype: str
        """
        for m in defs.MODEL_LIST:
            if m in self.model.upper():
                # sometimes las_vrs will be formatted like "EGM2008 height" and this should catch that
                self.model = m
                return True
        return False

    def get_height(self) -> float:
        """
        """
        if not self.verify_model():
            raise ValueError('No model found matching "%s"' % (self.model))
        if self.model in defs.NGS_MODELS:
            # format url for NGS API, then interpret json response
            ngs_model = defs.NGS_MODELS[self.model]
            self.url = utils.get_ngs_url(self.lat, self.lon, ngs_model)
            self.json = utils.get_ngs_json(self.url)
            self.height = float(self.json['geoidHeight'])
            return float(self.json['geoidHeight'])
        if self.model in defs.VDATUM_MODELS:
            # format url for VDatum API, then interpret json response
            if not self.region:
                self.region = defs.REGIONS[0]
                print('Warning: no region set. Defaulting to "%s"' % (self.region))
            else:
                self.region = self.region.lower()
            self.url = utils.get_vdatum_url(lat=self.lat, lon=self.lon, vdatum_model=self.model, region=self.region)
            self.json = utils.get_vdatum_json(self.url, self.region)
            self.height = float(self.json['t_z'])
            return float(self.json['t_z'])

    def in_feet_us_survey(self) -> float:
        """
        Convert to US survey foot (for imperial feet use `in_feet()`).
        1 imperial foot = 0.999998 US survey feet

        :return: Height value in usft (if it exists)
        :rtype: float
        """
        if self.height:
            return self.height * 3.2808333333

    def in_feet(self) -> float:
        """
        Convert to imperial foot (for US survey feet use `in_feet_us_survey()`).
        1 imperial foot = 0.999998 US survey feet

        :return: Height value in ft (if it exists)
        :rtype: float
        """
        if self.height:
            return self.height * 3.280839895
    
    def in_cm(self) -> float:
        """
        Convert to centimeters.

        :return: Height value in cm (if it exists)
        :rtype: float
        """
        if self.height:
            return self.height * 100

    def __str__(self) -> str:
        """
        Convert to string.

        :return: String representation of height or ``"n/a"``
        :rtype: str
        """
        if self.height:
            return '%s' % (self.height)
        else:
            return 'n/a'

    def as_string(self) -> str:
        """
        Convert to string using ``__str__``.

        :return:
        :rtype: str
        """
        return self.__str__()

    def available_models(self) -> list:
        """
        Return a list of available geoid and tidal models.

        :return: List of models
        :rtype: list
        """
        return defs.MODEL_LIST

    def available_regions(self) -> list:
        """
        Return a list of regions usable with VDatum models.
        See `VDatum regions <https://vdatum.noaa.gov/docs/services.html#step140>`_
        in the docs.

        Regions::

            ['contiguous', 'ak', 'seak', 'as', 'chesapeak_delaware',
            'westcoast', 'gcnmi', 'hi', 'prvi', 'sgi', 'spi', 'sli']

        :return: List of regions
        :rtype: list
        """
        return defs.REGIONS

    def __repr__(self) -> str:
        """
        Convert to printable representation, for example::

            >>> from pyegt.height import HeightModel
            >>> h = HeightModel(lat=64.506181, lon=-165.399282, from_model='EGM2008', region='ak')
            >>> repr(h)
            HeightModel(lat=64.506181, lon=-165.399282, from_model='EGM2008', region='ak') -> 6.998 meters

        :return: repr() value
        :rtype: str
        """
        if self.height:
            return "HeightModel(lat=%s, lon=%s, from_model='%s', region='%s') -> %s meters" % (self.lat, self.lon, self.model, self.region, self.height)
        else:
            return "HeightModel()"

    def __float__(self) -> float:
        """
        Convert to float.

        :return: Height value (if exists)
        :rtype: float
        """
        if self.height:
            return self.height
        else:
            raise ValueError('Cannot convert to float when no height has been calculated.')

    def as_float(self) -> float:
        """
        Convert to float using ``__float__``.

        :return: Height value (if exists)
        :rtype: float
        """
        return self.float()
    
    def __dict__(self) -> dict:
        """
        Return JSON object as dict.

        :return: JSON object
        :rtype: dict
        """
        if self.json:
            return self.json
        else:
            return None
        
    def as_dict(self) -> dict:
        """
        Return JSON object as dict using ``__dict__``.

        :return: JSON object
        :rtype: dict
        """
        return self.__dict__()

    def __eq__(self, __value: object) -> bool:
        """
        Use `math.isclose()` to compare as floating point numbers are not always directly comparable.

        :return: Boolean equality of two floats
        :rtype: bool
        """
        if self.height:
            return isclose(self.height, float(__value), rel_tol=1e-9, abs_tol=0.0)
        else:
            raise ValueError('Cannot compare when no height has been calculated.')



import json

from . import defs
from .height import HeightModel

def test():
    """
    Run tests.

    - Try GEOID12B with US, AK, and PR coordinates.

    - Try EGM2008 with US, AK, and PR coordinates.
    """
    TEST_COORDS = {
        defs.REGIONS[0]: {'lat': 44.256616, 'lon': -73.964784,}, # Jumping Complex, Lake Placid, NY
        defs.REGIONS[1]: {'lat': 64.486036, 'lon': -165.292154,}, # Nome River Bridge, Nome, AK
        defs.REGIONS[8]: {'lat': 18.471310, 'lon': -66.136544,}, # Isla de Cabras, Palo Seco, PR
    }
    MODELS = ['GEOID12B height',
              'EGM2008 height']
    TESTS = {}
    for m in MODELS:
        TESTS[m] = {}
        for t in TEST_COORDS:
            print('%s, %s' % (m, t))
            h = HeightModel(lat=TEST_COORDS[t]['lat'],
                            lon=TEST_COORDS[t]['lon'],
                            from_model=m, region=t)
            TESTS[m][t] = {'lookup': repr(h),
                            'height': float(h),
                            'result': None}
            if TESTS[m][t]['lookup']:
                repr(TESTS[m][t]['lookup'])
                TESTS[m][t]['result'] = True
            else:
                TESTS[m][t]['result'] = False
    js = json.dumps(TESTS, indent=2)
    print(js)



from pathlib import Path
from ._version import __version__
from datetime import datetime

Y = datetime.now().year
HELP_TXT = '''
~~~ pyegt version %s ~~~
 Ian Nesbitt / NCEAS %s
''' % (__version__, Y)

MOD_LOC = Path(__file__).parent.absolute()

NGS_MODELS = {
    # geoid model numbers in NGS API
    # from https://www.ngs.noaa.gov/web_services/geoid.shtml
    'GEOID99': 1,
    'G99SSS': 2,
    'GEOID03': 3,
    'USGG2003': 4,
    'GEOID06': 5,
    'USGG2009': 6,
    'GEOID09': 7,
    'XUSHG': 9,
    'USGG2012': 11,
    'GEOID12A': 12,
    'GEOID12B': 13,
    'GEOID18': 14,
}
""".. |ngsafg| raw:: html

    <a href="https://www.ngs.noaa.gov/web_services/geoid.shtml" target="_blank">NOAA NGS API for Geoid</a>

Geoid model numbers used in the NGS API.
From |ngsafg|.

Mapping::

    NGS_MODELS = {'GEOID99': 1, 'G99SSS': 2, 'GEOID03': 3, 'USGG2003': 4,
                  'GEOID06': 5, 'USGG2009': 6, 'GEOID09': 7, 'XUSHG': 9,
                  'USGG2012': 11, 'GEOID12A': 12, 'GEOID12B': 13, 'GEOID18': 14}
"""

NGS_URL = 'https://geodesy.noaa.gov/api/geoid/ght?lat=%s&lon=%s&model=%s'

VDATUM_MODELS = [
    # geoid models in VDatum API (US only)
    # from https://vdatum.noaa.gov/docs/services.html#step160
    'NAVD88', 'NGVD29', 'ASVD02', 'W0_USGG2012', 'GUVD04', 'NMVD03', 'PRVD02',
    'VIVD09', 'CRD', 'EGM2008', 'EGM1996', 'EGM1984', 'XGEOID16B', 'XGEOID17B',
    'XGEOID18B', 'XGEOID19B', 'XGEOID20B', 'IGLD85', 'LWD_IGLD85',
    'OHWM_IGLD85', 'CRD', 'LMSL', 'MLLW', 'MLW', 'MTL', 'DTL', 'MHW', 'MHHW',
    'LWD', 'NAD27', 'NAD83_1986', 'NAD83_2011', 'NAD83_NSRS2007',
    'NAD83_MARP00', 'NAD83_PACP00', 'WGS84_G1674', 'ITRF2014', 'IGS14',
    'ITRF2008', 'IGS08', 'ITRF2005', 'IGS2005', 'WGS84_G1150', 'ITRF2000',
    'IGS00', 'IGb00', 'ITRF96', 'WGS84_G873', 'ITRF94', 'ITRF93', 'ITRF92',
    'SIOMIT92', 'WGS84_G730', 'ITRF91', 'ITRF90', 'ITRF89', 'ITRF88',
    'WGS84_TRANSIT', 'WGS84_G1762', 'WGS84_G2139'
]
""".. |vdmodels| raw:: html

    <a href="https://vdatum.noaa.gov/docs/services.html#step160" target="_blank">VDatum API Vertical Reference Frames List</a>

List of geoid and tidal models used in the VDatum API. From |vdmodels|.
"""

REGIONS = [
    # VDatum regions (from https://vdatum.noaa.gov/docs/services.html#step140)
    'contiguous', 'ak', 'seak', 'as', 'chesapeak_delaware',
    'westcoast', 'gcnmi', 'hi', 'prvi', 'sgi', 'spi', 'sli'
]
""".. |regions| raw:: html

    <a href="https://vdatum.noaa.gov/docs/services.html#step140" target="_blank">VDatum API Regions List</a>

List of regions used in the VDatum API. From |regions|.
"""

VDATUM_URL = 'https://vdatum.noaa.gov/vdatumweb/api/convert?s_x=%s&s_y=%s&s_h_frame=%s&s_v_frame=%s&s_v_geoid=%s&t_h_frame=%s&t_v_frame=%s&t_v_geoid=%s&region=%s'

MODEL_LIST = []
for m in NGS_MODELS:
    MODEL_LIST.append(m)
for m in VDATUM_MODELS:
    MODEL_LIST.append(m)



import setuptools
from pyegt import _version

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    author='Ian Nesbitt',
    author_email='nesbitt@nceas.ucsb.edu',
    name='pyegt',
    version=_version.__version__,
    description='Look up geoid and tidal model heights relative to the ellipsoid',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://iannesbitt.github.io/pyegt',
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        'pyproj>=3.1.0',
        'requests'
    ],
    entry_points = {
        'console_scripts': [
            'pyegt-test=pyegt.test:test'
        ],
    },
    python_requires='>=3.9, <4.0',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering :: GIS',
    ],
    license='Apache Software License 2.0',
)